Object subclass: #TaalBendableSound
	instanceVariableNames: 'sound basePitch'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalBendableSound commentStamp: 'gp 7/15/2007 01:38' prior: 0!
Wraps a sound and adds pitch bending capability!


!TaalBendableSound methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 01:47'!
bend: aMidiPitchBend
	"Bends the sound by the given amount. 8192 means original pitch, 0 means - 1 octave and 16383 means + 1 octave"
	
	| bend factor |
	bend := (aMidiPitchBend - 8192) * 12.0 / 8192.0.
	factor := 	TaalBoxData pitchFactor: bend.
	
	sound pitch: basePitch * factor.
! !

!TaalBendableSound methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 01:58'!
doControl
	^ sound doControl.! !

!TaalBendableSound methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 02:01'!
doesNotUnderstand: aMessage 
self halt.
	Transcript show: 'Bendable: ', aMessage asString; cr.! !

!TaalBendableSound methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 01:59'!
mixSampleCount: n into: aSoundBuffer startingAt: startIndex leftVol: leftVol rightVol: rightVol
	^ sound mixSampleCount: n into: aSoundBuffer startingAt: startIndex leftVol: leftVol rightVol: rightVol.! !

!TaalBendableSound methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 01:45'!
on: aSound pitch: aPitch
	sound := aSound.
	basePitch := aPitch.! !

!TaalBendableSound methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 01:58'!
samplesRemaining
	^ sound samplesRemaining.! !


Object subclass: #TaalBox
	instanceVariableNames: 'taalPlayer dronePlayer lehras lehra thekas theka drones drone availableLehras availableThekas globalVolume lehraVolume thekaVolume droneVolume pulse dronePulse baseNote fineTune lehraInstrument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalBox commentStamp: 'gp 7/2/2007 17:02' prior: 0!
Lehras and thekas are collections of collections of midi scores: Dictionary<taal, Collection<TaalScore>>
Drones is a collection of scores: Collection<TaalScore>

availbleThekas and availableLehras are the thekas and lehras that are available for the current taal.!


!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:29'!
allPlayers
	^ Array with: taalPlayer with: dronePlayer.

! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 01:57'!
availableDrones
	"Returns a collection of TaalScore representing all available drones"
	^ drones.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 01:56'!
availableLehras
	"Returns a collection of TaalScore representing all available lehras for the current taal"
	^ availableLehras.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 01:56'!
availableThekas
	"Returns a collection of TaalScore representing all available thekas for the current taal"
	^ availableThekas.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 16:04'!
dispatch: aSelector to: aTargets
	self dispatch: aSelector withArguments: #() to: aTargets.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 16:02'!
dispatch: aSelector withArguments: aArgs to: aTargets
	aTargets do: [ :t |
		t ifNotNil: [ t perform: aSelector withArguments: aArgs]].! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 16:31'!
dronePulse
	^ dronePulse.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 17:23'!
hasScores: aTaal
	"Indicates if there are score (both lehra and theka) for the given taal"
	| scores |
	
	scores := lehras at: aTaal ifAbsent: [ ^ false ].
	scores isEmptyOrNil ifTrue: [ ^ false ].
	
	scores := thekas at: aTaal ifAbsent: [ ^ false ].
	scores isEmptyOrNil ifTrue: [ ^ false ].

	^ true.	! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 8/18/2007 16:59'!
initialize
	self reload.
	
	taalPlayer := TaalScorePlayer new disableReverb: true.
	dronePlayer := TaalScorePlayer new setDurationMultiplier: 9.0.
	
	globalVolume := 1.0.	
	lehraVolume := 1.0.	
	thekaVolume := 1.0.	
	droneVolume := 1.0.	
	self updateVolume.
	
	self setPulse: 120.
	self setDronePulse: 60.
	! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:59'!
loadTaalCollection: aDir
	"Loads the tracks of the given directory (returns a Collection of TaalScoreInfo)"
	| list dir file score |
	
	list := OrderedCollection new.
	dir := FileDirectory default on: aDir.
	
	dir fileNames asSortedCollection do: [ :n |
		(n endsWith: '.mid') ifTrue: [
			file := dir fullNameFor: n.
			score := TaalScorePlayer openScore: file.
			list add: (TaalScoreInfo new name: n midiScore: score).
		]
	].
	
	^ list.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'spfa 7/7/2015 12:50'!
loadTaalCollections: aDir
	"Loads the tracks of the given directory (organized by taal)"
	| dict base dir file scores |
	
	dict := Dictionary new.
	base := FileDirectory default fullNameFor: ('taalbox', FileDirectory pathNameDelimiter, 'snd', FileDirectory pathNameDelimiter, 'tracks', FileDirectory pathNameDelimiter, aDir).
	dir := FileDirectory default on: base.
	
	dir directoryNames do: [ :n |
		file := dir fullNameFor: n.
		scores := self loadTaalCollection: file.
		dict at: (n asInteger) put: scores.
	].
	
	^ dict.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 20:28'!
makeCompoundScore: aScores
	"Creates a compound score with all the given scores.
	For now the tabla score should come first, as track 4 is considered a percussion track."
	
	| refScore tracks score |
	
	tracks := OrderedCollection new.
	aScores do: [ :s |
		s ifNil: [ ^ nil ].
		s midiScore tracks do: [ :t | tracks add: t ].
	].

	refScore := (aScores at: 1) midiScore.

	score := TaalScore new
		tracks: tracks;
		trackInfo: #('Compound score');
		tempoMap: (refScore tempoMap);
		ticksPerQuarterNote: (refScore ticksPerQuarterNote).

	^ TaalScoreInfo new name: 'taal' midiScore: score.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/9/2007 15:03'!
play
	SoundPlayer initialize; shutDown; startUp.
	self dispatch: #play to: (self allPlayers).
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:59'!
pulse
	^ pulse.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'spfa 7/7/2015 12:50'!
reload
	"Reload all tracks"
	
	lehras := self loadTaalCollections: 'lehra'.
	thekas := self loadTaalCollections: 'theka'.	
	drones := self loadTaalCollection: (FileDirectory default fullNameFor: 'taalbox', FileDirectory pathNameDelimiter, 'snd', FileDirectory pathNameDelimiter, 'tracks', FileDirectory pathNameDelimiter, 'drone').! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 02:03'!
setBaseNote: aNote
	baseNote := aNote.
	self dispatch: #setBaseNote: withArguments: (Array with: aNote) to: (self allPlayers).
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:38'!
setDrone: aScore
	| wasPlaying |
	
	wasPlaying := dronePlayer isPlaying.
	wasPlaying ifTrue: [ dronePlayer stop ].
	drone := aScore.
	self setScore: aScore on: dronePlayer.
	dronePlayer setInstrument: 1 to: (TaalBoxData tampura).	
	wasPlaying ifTrue: [ dronePlayer play ].

! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 16:31'!
setDronePulse: aPulse
	dronePulse := aPulse.
	dronePlayer setPulse: dronePulse.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:44'!
setDroneVolume: aVolume
	"Sets the volume of the drone player [0..1]"
	droneVolume := aVolume.
	self updateVolume.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 02:03'!
setFineTune: aFineTune
	fineTune := aFineTune.
	self dispatch: #setFineTune: withArguments: (Array with: aFineTune) to: (self allPlayers).! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:43'!
setLehra: aScore
	lehra := aScore.
	self setupTaalPlayer.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:37'!
setLehraInstrument: aInstrument
	"Sets the instrument of the lehra player"
	lehraInstrument := aInstrument.
	taalPlayer setInstrument: 5 to: lehraInstrument.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:43'!
setLehraVolume: aVolume
	"Sets the volume of the lehra player [0..1]"
	lehraVolume := aVolume.
	self updateVolume.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 02:03'!
setPulse: aPulse
	pulse := aPulse.
	self dispatch: #setPulse: withArguments: (Array with: aPulse) to: (self taalPlayers).
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 8/18/2007 16:58'!
setScore: aScore on: aPlayer
	| wasPlaying |
	wasPlaying := aPlayer isPlaying.
	wasPlaying ifTrue: [ aPlayer stop ].
	aPlayer onScore: (aScore midiScore).
	aPlayer reset.
	(aPlayer = dronePlayer) ifTrue: [ aPlayer setPulse: dronePulse ] ifFalse: [ aPlayer setPulse: pulse ].
	aPlayer setBaseNote: baseNote.
	aPlayer setFineTune: fineTune.
	wasPlaying ifTrue: [ aPlayer play ].! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 16:57'!
setTaal: aBeatCount
	availableLehras := lehras at: aBeatCount.
	availableThekas := thekas at: aBeatCount.! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:42'!
setTheka: aScore
	theka := aScore.
	self setupTaalPlayer.
	! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:43'!
setThekaVolume: aVolume
	"Sets the volume of the lehra player [0..1]"
	thekaVolume := aVolume.
	self updateVolume.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:43'!
setVolume: aVolume
	"Sets the global volume [0..1]"
	globalVolume := aVolume.
	self updateVolume.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:37'!
setupTaalPlayer
	"Sets up the taal player to a score that is compound of the lehra and theka scores"
	
	| wasPlaying score |
	wasPlaying := taalPlayer isPlaying.
	self stop.
	
	score := self makeCompoundScore: (Array with: theka with: lehra).
	score ifNil: [ ^ self ].
	self setScore: score on: taalPlayer.
	
	1 to: (TaalBoxData tabla size) do: [ :i | 
		taalPlayer setInstrument: i to: (TaalBoxData tabla at: i).
		taalPlayer panForTrack: i put: 0.5. ].	
		
	taalPlayer setInstrument: 5 to: lehraInstrument.
	taalPlayer panForTrack: 5 put: 0.5.	

	self updateVolume.
	wasPlaying ifTrue: [ self play ].
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 16:18'!
stop
	self dispatch: #stop to: (self allPlayers).
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:29'!
taalPlayers
	^ Array with: taalPlayer.
! !

!TaalBox methodsFor: 'as yet unclassified' stamp: 'gp 7/11/2007 17:38'!
updateVolume
	"Sets the volume of all players."
	taalPlayer overallVolume: (globalVolume * 0.8).
	dronePlayer overallVolume: (globalVolume * droneVolume * 0.8).

	(taalPlayer trackCount = 5) ifTrue: [
		"Only set up the volume if both theka and lehra are present"
		taalPlayer volumeForTrack: 1 put: (thekaVolume / 4).
		taalPlayer volumeForTrack: 2 put: (thekaVolume / 4).
		taalPlayer volumeForTrack: 3 put: (thekaVolume / 4).
		taalPlayer volumeForTrack: 4 put: (thekaVolume / 4).
		taalPlayer volumeForTrack: 5 put: (lehraVolume / 4).
	].! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaalBox class
	instanceVariableNames: ''!

!TaalBox class methodsFor: 'as yet unclassified' stamp: 'spfa 1/7/2011 13:29'!
open

	"entry point with initialization - spfa
	
	TaalBox open
	"
	
	TaalBoxData load.
	TaalBoxMorph2 restart! !

!TaalBox class methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:26'!
readInstrumentFrom: aDir 
	^ self readInstrumentFrom: aDir name: aDir pitchBendStyle: #none.! !

!TaalBox class methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:25'!
readInstrumentFrom: aDir name: aName pitchBendStyle: aStyle
	"Reads an instrument from a sample set directory containing AIFF files"

	| sampleSet snd dir |
	dir := FileDirectory default fullNameFor: aDir.
	sampleSet := self readSampleSetFrom: dir.
	snd := TaalSampledInstrument new allSampleSets: sampleSet.
	
	snd name: aName pitchBendStyle: aStyle.

	"fix slow attacks"
	snd allNotes do: [:n |
		n firstSample: (n findStartPointForThreshold: 1000)].

	^ snd
! !

!TaalBox class methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 13:11'!
readSampleSetFrom: dirName
	"Answer a collection of sounds read from AIFF files in the given directory and sorted in ascending pitch order.
	Copied and adapted from SampledInstrument."

	| all dir fullName snd |
	all := SortedCollection sortBlock: [:s1 :s2 | s1 pitch < s2 pitch].
	dir := FileDirectory default on: dirName.
	dir fileNames do: [:n |
		fullName := dir fullNameFor: n.
		(fullName endsWithAnyOf: #('.aiff')) ifTrue: [
			snd := LoopedSampledSound new
				fromAIFFFileNamed: fullName
				mergeIfStereo: true.
			all add: snd]].
	^ all asArray
! !


Object subclass: #TaalBoxData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalBoxData commentStamp: 'gp 7/8/2007 18:04' prior: 0!
Permits to maintain some static data (instruments, icons) of TaalBox loaded in the image!


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaalBoxData class
	instanceVariableNames: 'instrTampura instrTabla instrMelodic pitchFactors iconPulse iconTuning iconVolume iconTaal iconInstrument iconSystem iconPlay iconStop iconMute iconUnmute iconVolLehra iconVolTheka iconVolDrone iconVolGlobal'!

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 23:28'!
iconInstrument
	^ iconInstrument.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 22:10'!
iconMute
	^ iconMute.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 19:28'!
iconPlay
	^ iconPlay.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:12'!
iconPulse
	^ iconPulse.
! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 19:28'!
iconStop
	^ iconStop.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:13'!
iconSystem
	^ iconSystem.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:13'!
iconTaal
	^ iconTaal.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:12'!
iconTuning
	^ iconTuning.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:14'!
iconUnmute
	^ iconUnmute.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:02'!
iconVolDrone
	^ iconVolDrone.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:02'!
iconVolGlobal
	^ iconVolGlobal.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:02'!
iconVolLehra
	^ iconVolLehra.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:02'!
iconVolTheka
	^ iconVolTheka.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:13'!
iconVolume
	^ iconVolume.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'spfa 1/7/2011 11:54'!
load
	instrTampura := TaalBox readInstrumentFrom: 'taalbox\snd\instruments\tampura'.
	
	instrMelodic := Array 
		with: (TaalBox readInstrumentFrom: 'taalbox\snd\instruments\melodica' name: 'Melodica' pitchBendStyle: #discreet)
		with: (TaalBox readInstrumentFrom: 'taalbox\snd\instruments\Guillaume' name: 'Guillaume' pitchBendStyle: #continuous).
		
	
	instrTabla := Array with: (TaalBox readInstrumentFrom: 'taalbox\snd\instruments\tabla\Na')
		with: (TaalBox readInstrumentFrom: 'taalbox\snd\instruments\tabla\Tin')
		with: (TaalBox readInstrumentFrom: 'taalbox\snd\instruments\tabla\Tun')
		with: (TaalBox readInstrumentFrom: 'taalbox\snd\instruments\tabla\other').

	"Precompute pitch factors"
	pitchFactors := Dictionary new.
	-1200 to: 1200 do: [ :i |
		pitchFactors at: i put: (1.059463 raisedTo: (i asFloat / 100)).
	].

	"Load icons"
	iconPulse := Form fromFileNamed: 'taalbox\metronome.png'.
	iconTuning := Form fromFileNamed: 'taalbox\tuningfork.png'.	
	iconTaal := Form fromFileNamed: 'taalbox\taal.png'.	
	iconVolume := Form fromFileNamed: 'taalbox\icon-volume.png'.	
	iconInstrument := Form fromFileNamed: 'taalbox\icon-instrument.png'.	
	iconSystem := Form fromFileNamed: 'taalbox\system.png'.	
	
	iconPlay := Form fromFileNamed: 'taalbox\play.png'.
	iconStop := Form fromFileNamed: 'taalbox\stop.png'.
	iconMute := Form fromFileNamed: 'taalbox\mute.png'.
	iconUnmute := Form fromFileNamed: 'taalbox\unmute.png'.

	iconVolLehra := Form fromFileNamed: 'taalbox\harmonium.png'.
	iconVolTheka := Form fromFileNamed: 'taalbox\tabla.png'.
	iconVolDrone := Form fromFileNamed: 'taalbox\tampura.png'.
	iconVolGlobal := Form fromFileNamed: 'taalbox\volume.png'.
! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/19/2007 00:59'!
melodic
	^ instrMelodic.! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 01:42'!
pitchFactor: aFineTune
	"Returns the amount to multiply a pitch to adjust to the specified fine tune. +1.0 means + 1 semitone,
	-1.0 means - 1 semitone."
	
	| i |
	
	i := (aFineTune * 100) asInteger.
	^ pitchFactors at: i ifAbsent: [ ^ 1.0 ].! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:05'!
tabla
	^ instrTabla.

! !

!TaalBoxData class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:05'!
tampura
	^ instrTampura.

! !


AlignmentMorph subclass: #TaalBoxMorph
	instanceVariableNames: 'taalBox pulseText pulseSlider pulse pianoMorph baseNote fineTuneSlider fineTune taalSelectorMorph thekaSelectorMorph lehraSelectorMorph droneSelectorMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalBoxMorph commentStamp: 'gp 7/6/2007 10:35' prior: 0!
Usage:
TaalBoxInstruments load.
TaalBoxMorph restart.
!


!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 02:17'!
createDroneSelector
	"Creates a control that permits to select an available drone"
	
	droneSelectorMorph := TaalScoreSelectorMorph new
		actionSelector: #setDrone:.
		
	^ droneSelectorMorph.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 02:17'!
createLehraSelector
	"Creates a control that permits to select an available lehra for the current taal"
	
	lehraSelectorMorph := TaalScoreSelectorMorph new
		actionSelector: #setLehra:.
		
	^ lehraSelectorMorph.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:49'!
createPitchControl
	| col r1 r2 |
	
	col := self makeColumn.
	
	r1 := TaalPane makeRow
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		height: 24.
		
	r2 := TaalPane makeRow
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		height: 24.
		
	col addMorphBack: r1.
	col addMorphBack: r2.
	
	pianoMorph := TaalPianoMorph new
		target: self;
		noteOnSelector: #setBaseNote:.
		
	r1 addMorphBack: pianoMorph.
	
	fineTuneSlider := SimpleSliderMorph new
		extent: 80 @ 2;
		target: [ :v | self setFineTune: v];
		actionSelector: #value:;
		minVal: -1.0;
		maxVal: 1.0.
	
	r2 addMorphBack: (StringMorph contents: '-').
	r2 addMorphBack: fineTuneSlider.
	r2 addMorphBack: (StringMorph contents: '+').
	
	r2 addMorphBack: (self makeButton
		label: '0';
		target: [ self setFineTune: 0 ]; 
		actionSelector: #value).
	
	^ col
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:49'!
createPlayControl
	| r |
	
	r := TaalPane makeRow
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		height: 24.

	r addMorphBack: (self makeButton
		label: 'Play';
		target: [ taalBox play ];
		actionSelector: #value).
		
	r addMorphBack: (self makeButton
		label: 'Stop';
		target: [ taalBox stop ];
		actionSelector: #value).
		
	r addMorphBack: (TaalVolumeControlMorph new
		target: [ :v | taalBox setVolume: v ];
		actionSelector: #value:).
		
	^ r
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:49'!
createPulseControl
	| col r1 r2 |
	
	col := TaalPane makeColumn.
	
	r1 := TaalPane makeRow
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		height: 24.
	
	r2 := TaalPane makeRow
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		height: 24.
	
	pulseSlider := SimpleSliderMorph new
		extent: 120 @ 2;
		target: self;
		actionSelector: #setPulse:;
		minVal: 20;
		maxVal: 240.
		
	pulseText := StringMorph contents: '?'.
	pulseText emphasis: 1.
		

	r1 addMorphBack: (self makeButton 
		label: '-';
		target: [ self setPulse: (pulse - 1) ]; 
		actionSelector: #value).
	
	r1 addMorphBack: (self makeButton 
		label: '+';
		target: [ self setPulse: (pulse + 1) ]; 
		actionSelector: #value).
	
	r1 addMorphBack: (StringMorph contents: ' ').
	r1 addMorphBack: (StringMorph contents: (pulseSlider minVal asString), ' ').
	r1 addMorphBack: pulseSlider.
	r1 addMorphBack: (StringMorph contents: ' ', (pulseSlider maxVal asString)).
	
	col addMorphBack: r1.
	
	r2 addMorphBack: (self makeButton 
		label: '/2';
		target: [ self setPulse: (pulse / 2) ]; 
		actionSelector: #value).
	
	r2 addMorphBack: (StringMorph contents: ' ').
	r2 addMorphBack: pulseText.
	r2 addMorphBack: (StringMorph contents: ' ').

	r2 addMorphBack: (self makeButton
		label: 'x2';
		target: [ self setPulse: (pulse * 2) ]; 
		actionSelector: #value).
	
	col addMorphBack: r2.
	
	^ col
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:50'!
createSystemControl
	| r |
	
	r := TaalPane makeRow
		hResizing: #shrinkWrap;
		vResizing: #rigid;
		height: 24.

	r addMorphBack: (self makeButton
		label: 'Reload';
		target: [ taalBox reload ];
		actionSelector: #value).		
				
	r addMorphBack: (self makeButton
		label: 'Restart';
		target: [ TaalBoxMorph restart ];
		actionSelector: #value).		
				
	r addMorphBack: (self makeButton
		label: 'Quit';
		target: [ SmalltalkImage current quitPrimitive ];
		actionSelector: #value).		
				
	^ r.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 01:38'!
createTaalSelector
	"Creates a control that permits to select the number of beats (rather than concrete taal).
	The actual morphs are set up in updateTaalSelector"
	
	taalSelectorMorph := self makeColumn.
	^ taalSelectorMorph.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 02:17'!
createThekaSelector
	"Creates a control that permits to select an available theka for the current taal"
	
	thekaSelectorMorph := TaalScoreSelectorMorph new
		actionSelector: #setTheka:.
		
	^ thekaSelectorMorph.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 00:35'!
createVolumeControl: aSelector
	"Creates a control that permits to set the volume of a channel.
	The given selector will be called on taalBox with the volume as an argument"
	
	^ TaalVolumeControlMorph new
		target: [ :v | taalBox perform: aSelector with: v ];
		actionSelector: #value:.
		! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:50'!
initialize
	| r |
	super initialize.
	
	self beSticky.

	self listDirection: #topToBottom;
		 wrapCentering: #center;
		 cellPositioning: #topCenter;
		 hResizing: #shrinkWrap;
		 vResizing: #shrinkWrap;
		 layoutInset: 3;
		 extent: 320 @ 240.

	self addMorphBack: self createSystemControl.
	self addMorphBack: self createPlayControl.
	self addMorphBack: self createPulseControl.
	self addMorphBack: self createPitchControl.
	self addMorphBack: self createTaalSelector.

	r := TaalPane makeRow.	
	r addMorphBack: self createLehraSelector.
	r addMorphBack: (self createVolumeControl: #setLehraVolume:).
	self addMorphBack: r.

	r := TaalPane makeRow.	
	r addMorphBack: self createThekaSelector.
	r addMorphBack: (self createVolumeControl: #setThekaVolume:).
	self addMorphBack: r.

	r := TaalPane makeRow.	
	r addMorphBack: self createDroneSelector.
	r addMorphBack: (self createVolumeControl: #setDroneVolume:).
	self addMorphBack: r.

	
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 01:44'!
makeButton
	^ TaalBoxMorph makeButton.
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/1/2007 18:03'!
makeColumn 

	^ AlignmentMorph newColumn
		color: color;
		layoutInset: 0;
		wrapCentering: #center; 
		cellPositioning: #leftCenter;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/1/2007 18:03'!
makeRow

	^ AlignmentMorph newRow
		color: color;
		layoutInset: 0;
		wrapCentering: #center; 
		cellPositioning: #leftCenter;
		hResizing: #spaceFill;
		vResizing: #shrinkWrap
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 21:15'!
on: aTaalBox
	taalBox := aTaalBox.
	
	self setPulse: 60.
	self setBaseNote: 60.
	self setFineTune: 0.0.
	
	lehraSelectorMorph target: taalBox.
	thekaSelectorMorph target: taalBox.
	droneSelectorMorph target: taalBox.

	self updateTaalSelector.
	self setTaal: 16.	
	
	! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 15:29'!
setBaseNote: aNote
	(baseNote = aNote) ifTrue: [ ^ self ].
	
	baseNote := aNote.
	pianoMorph setKey: baseNote.
	taalBox setBaseNote: baseNote.
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 15:30'!
setFineTune: aFineTune
	(fineTune = aFineTune) ifTrue: [ ^ self ].
	
	fineTune := aFineTune.
	fineTuneSlider adjustToValue: fineTune.
	taalBox setFineTune: aFineTune.
! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 15:31'!
setPulse: aPulse
	(pulse = aPulse) ifTrue: [ ^ self ].

	pulse := (aPulse asInteger) min: (pulseSlider maxVal) max: (pulseSlider minVal).
	pulseText contents: (pulse asString).
	pulseSlider adjustToValue: pulse.
	taalBox setPulse: aPulse.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 02:01'!
setTaal: aTaal
	taalBox setTaal: aTaal.
	self updateScoreSelectors.! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 02:26'!
updateScoreSelectors
	lehraSelectorMorph on: (taalBox availableLehras).
	thekaSelectorMorph on: (taalBox availableThekas).
	droneSelectorMorph on: (taalBox availableDrones).! !

!TaalBoxMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:50'!
updateTaalSelector
	"Creates a control that permits to select the number of beats (rather than concrete taal)"
	| col row n |
	
	col := taalSelectorMorph.
	col removeAllMorphs.
	
	n := 0.
	
	1 to: 20 do: [ :i |
		(taalBox hasScores: i) ifTrue: [
		
			(n = 0) ifTrue: [
				row := TaalPane makeRow.
				col addMorphBack: row.
				n := 4.
			].

			row addMorphBack: (self makeButton
				label: (i asString);
				extent: 30 @ 30;
				target: self;
				actionSelector: #setTaal:;
				arguments: (Array with: i)).
			
			n := n-1.
		]
	].
	
	^ col
! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaalBoxMorph class
	instanceVariableNames: ''!

!TaalBoxMorph class methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 01:44'!
makeButton
	^ SimpleButtonMorph new.
! !

!TaalBoxMorph class methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 15:31'!
restart
	"Removes all existing TaalBoxMorphs and spawns a new one."
	
	TaalBox allSubInstancesDo: [ :tb | tb stop ].
	
	TaalBoxMorph allSubInstancesDo: [ :m |
		m delete.
		Transcript cr; show: 'Destroyed a TaalBoxMorph'.
	].
	"FileStream fileIn: 'TaalBox.st'."
	(TaalBoxMorph new on: (TaalBox new)) openInWorld.
! !


RectangleMorph subclass: #TaalBoxMorph2
	instanceVariableNames: 'taalBox panes currentMorph highlight p1 p2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalBoxMorph2 commentStamp: 'gp 8/18/2007 16:58' prior: 0!
Usage:
TaalBoxData load.
TaalBoxMorph2 restart.!


!TaalBoxMorph2 methodsFor: 'as yet unclassified' stamp: 'gp 7/9/2007 00:25'!
highlight: aButton
	highlight position: aButton position; extent: aButton extent; goBehind.! !

!TaalBoxMorph2 methodsFor: 'as yet unclassified' stamp: 'gp 7/13/2007 19:20'!
initialize
	| y b |
	super initialize.
	self beSticky.

	self extent: 240 @ 320.
	self borderWidth: 0.
	
	highlight := RectangleMorph new 
		color: Color yellow;
		borderWidth: 0.
		
	self addMorphBack: highlight.
	
	panes := OrderedCollection new.
	y := 45.
	self panes do: [ :p |
		panes add: p.
		p color: Color white.
		
		b := ThreePhaseButtonMorph new.
		b target: [ :b0 :p0 |
				self showMorph: p0.
				self highlight: b0.
			];
			actionSelector: #value:value:;
			arguments: (Array with:b with: p);
			onImage: p icon;
			offImage: p icon;
			position: 0 @ y;
			extent: 32 @ 32.
			
		self addMorphBack: b.
		self addMorphBack: p.
		p hide.
		p position: 34@40.
		
		currentMorph ifNil: [ self showMorph: p; highlight: b ].
		
		y := y+40.
	].

	b := ThreePhaseButtonMorph new 
		target: [ taalBox play ];
		actionSelector: #value;
		onImage: TaalBoxData iconPlay;
		offImage: TaalBoxData iconPlay;
		position: 70 @ 0;
		extent: 40 @ 40.
		
	self addMorphBack: b.

	b := ThreePhaseButtonMorph new 
		target: [ taalBox stop ];
		actionSelector: #value;
		onImage: TaalBoxData iconStop;
		offImage: TaalBoxData iconStop;
		position: 130 @ 0;
		extent: 40 @ 40.
		
	self addMorphBack: b.

	self color: Color white.
	
	self addMorphBack: (RectangleMorph new color: (Color black);
		position: 32@40;
		extent: 2@280).
		
	self addMorphBack: (RectangleMorph new color: (Color black);
		position: 0@38;
		extent: 240@2).	
		
	self addMorphFront: (StringMorph new contents: 'http://taalpulse.sourceforge.net/';
		position: 45@305;
		extent: 240@20).! !

!TaalBoxMorph2 methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 19:10'!
on: aTaalBox
	taalBox := aTaalBox.
	panes do: [ :p | p on: aTaalBox ].! !

!TaalBoxMorph2 methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 23:29'!
panes
	"Returns all the available panes"
	
	^ Array 
		with: (TaalVolumePane new)	
		with: (TaalPulsePane new) 
		with: (TaalScorePane new)
		with: (TaalInstrumentPane new)
		with: (TaalPitchPane new)
		with: (TaalSystemPane new).
		! !

!TaalBoxMorph2 methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:23'!
showMorph: aMorph
	currentMorph ifNotNil: [ currentMorph hide ].
	currentMorph := aMorph. 
	currentMorph ifNotNil: [ currentMorph show ].
! !

!TaalBoxMorph2 methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:57'!
taalBox
	^ taalBox.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaalBoxMorph2 class
	instanceVariableNames: ''!

!TaalBoxMorph2 class methodsFor: 'as yet unclassified' stamp: 'gp 8/18/2007 16:57'!
restart
	"Removes all existing TaalBoxMorphs and spawns a new one."
	
	Preferences disable: #warnIfNoSourcesFile.
	Preferences disable: #warnIfNoChangesFile.
	
	SoundPlayer initialize; shutDown; startUp.
	TaalBox allSubInstancesDo: [ :tb | tb stop ].
	
	TaalBoxMorph2 allSubInstancesDo: [ :m |
		m delete.
		Transcript cr; show: 'Destroyed a TaalBoxMorph2'.
	].

	(TaalBoxMorph2 new on: (TaalBox new)) openInWorld.
! !


SimpleButtonMorph subclass: #TaalButtonMorph
	instanceVariableNames: 'reclick'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalButtonMorph commentStamp: 'gp 7/11/2007 17:44' prior: 0!
This button has an improved mouseStillDown behavior.!


!TaalButtonMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/12/2007 22:15'!
mouseDown: evt
	reclick := 0.
	super mouseDown: evt.! !

!TaalButtonMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/12/2007 22:18'!
mouseStillDown: evt
	reclick := reclick +1.
	(reclick == 1) ifTrue: [ super mouseStillDown: evt ].
	(reclick > 3) ifTrue: [ super mouseStillDown: evt ].	! !


RectangleMorph subclass: #TaalPane
	instanceVariableNames: 'image taalBox'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalPane commentStamp: 'gp 7/8/2007 19:16' prior: 0!
"A pane in the TaalBox. It is a regular morph of fixed dimensions and position. Additionally
it contains an image that is used to select it in the TaalBox"!


!TaalPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 17:58'!
icon
	"Returns the icon of this pane".
	self subclassResponsibility.! !

!TaalPane methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 22:11'!
initialize
	super initialize.
	
	self extent: 206@265;
		hResizing: #rigid; 
		vResizing: #rigid.
	
	self borderWidth: 0.
	image := self icon.
	self color: Color white.! !

!TaalPane methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 12:49'!
makeTitle: aTitle
	^  StringMorph new 
		font: (TextStyle default fontOfSize: 30);
		contents: aTitle.! !

!TaalPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 19:01'!
on: aTaalBox
	taalBox := aTaalBox.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaalPane class
	instanceVariableNames: 'buttonColor'!

!TaalPane class methodsFor: 'as yet unclassified' stamp: 'gp 7/12/2007 22:12'!
buttonColor
	"Returns the default button color"
	buttonColor ifNil: [ buttonColor := Color r: 0.4 g: 0.8 b: 0.6 ].
	^ buttonColor.! !

!TaalPane class methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:34'!
makeButton
	^ SimpleButtonMorph new
		hResizing: #rigid; 
		vResizing: #rigid;
		color: Color blue;
		extent: 32@32.
! !

!TaalPane class methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:46'!
makeColumn 

	^ AlignmentMorph newColumn
		color: Color white;
		layoutInset: 0;
		wrapCentering: #center; 
		cellPositioning: #leftCenter;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap
! !

!TaalPane class methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:46'!
makeRow

	^ AlignmentMorph newRow
		color: Color white;
		layoutInset: 0;
		wrapCentering: #center; 
		cellPositioning: #leftCenter;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap
! !

!TaalPane class methodsFor: 'as yet unclassified' stamp: 'gp 7/12/2007 22:13'!
makeTextButton: aText
	^ TaalButtonMorph new
		borderColor: #raised;
		borderWidth: 2;
		color: self buttonColor;
		label: aText;
		extent: 32@32.
! !


TaalPane subclass: #TaalInstrumentPane
	instanceVariableNames: 'lehraInstrumentSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalInstrumentPane methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 23:28'!
icon
	^ TaalBoxData iconInstrument.! !

!TaalInstrumentPane methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:36'!
initialize
	| c |
	super initialize.
	
	self layoutPolicy: TableLayout new.
	self listDirection: #topToBottom;
		wrapCentering: #topCenter;
		cellPositioning: #topCenter;
		layoutInset: 3;
		cellInset: 3;
		hResizing: #spaceFill.
		
	self addMorphBack: (self makeTitle: 'Instruments').

	c := TaalPane makeColumn
		hResizing: #spaceFill;
		wrapCentering: #topLeft.
		
	self addMorphBack: c.
	
	lehraInstrumentSelector := TaalInstrumentSelectorMorph new
		actionSelector: #setLehraInstrument:.
		
	lehraInstrumentSelector setTitle: 'Lehra'.

	c addMorphBack: lehraInstrumentSelector.
! !

!TaalInstrumentPane methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:35'!
on: aTaalBox
	super on: aTaalBox.

	lehraInstrumentSelector target: taalBox.
	lehraInstrumentSelector on: (TaalBoxData melodic).
! !


RectangleMorph subclass: #TaalPianoMorph
	instanceVariableNames: 'whiteKeyColor blackKeyColor playingKeyColor nOctaves target noteOnSelector morphDictionary currentKeyMorph oldColor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalPianoMorph commentStamp: 'gp 7/2/2007 00:09' prior: 0!
Copied and adapted from PianoKeyboardMorph!


!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 16:11'!
buildKeyboard
	| wtWid bkWid keyRect octavePt nWhite nBlack morph pitch baseOctave p |
	self removeAllMorphs.
	baseOctave := 3.
	morphDictionary := Dictionary new.
	wtWid _ 12. bkWid _ 8.
	self extent: 10@10.
	1 to: nOctaves+1 do:
		[:i | i <= nOctaves ifTrue: [nWhite _ 7.  nBlack _ 5]
						ifFalse: [nWhite _ 1.  nBlack _ 0 "High C"].
		octavePt _ self innerBounds topLeft + ((7*wtWid*(i-1)-1)@-1).
		1 to: nWhite do:
			[:j | keyRect _ octavePt + (j-1*wtWid@0) extent: (wtWid+1)@54.
			pitch := ((i+baseOctave)*12) - 1 + (#(1 3 5 6 8 10 12) at: j).
			morph := (RectangleMorph newBounds: keyRect color: whiteKeyColor)
							borderWidth: 1;
							on: #mouseDown send: #mouseDownPitch:event:noteMorph: to: self withValue: pitch.
			morphDictionary at: pitch put: morph.
			self addMorph: morph].
		
		1 to: nBlack do: [:j | 
			p := ((#(1 2 4 5 6) at: j) * wtWid) - (bkWid / 2).
			keyRect _ octavePt + (p@1) extent: bkWid@32.
			pitch := ((i+baseOctave)*12) - 1 + (#(2 4 7 9 11) at: j).
			morph := (Morph newBounds: keyRect color: blackKeyColor)
							on: #mouseDown send: #mouseDownPitch:event:noteMorph: to: self withValue: pitch.
			morphDictionary at: pitch put: morph.
			self addMorph: morph]].
	
	self submorphsDo:
		[:m | m on: #mouseMove send: #mouseMovePitch:event:noteMorph: to: self;
				on: #mouseEnterDragging send: #mouseDownPitch:event:noteMorph: to: self.].
	
	self extent: (self fullBounds extent + borderWidth - 1).! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 00:08'!
defaultColor
	"answer the default color/fill style for the receiver"
	^ Color veryLightGray! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:35'!
initialize
	super initialize.

	whiteKeyColor:= Color gray: 0.95.
	blackKeyColor := Color black.
	playingKeyColor := Color blue.
	nOctaves := 2.
	self buildKeyboard.
! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 00:33'!
mouseDownPitch: midiKey event: event noteMorph: noteMorph
	event hand hasSubmorphs ifTrue: [^ self  "no response if drag something over me"].
	event hand mouseFocus ifNil:
		["If dragged into me, then establish focus so I'll see moves"
		event hand newMouseFocus: noteMorph event: event].

	self setKey: midiKey.! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 00:32'!
mouseMovePitch: pitch event: event noteMorph: noteMorph

	(noteMorph containsPoint: event cursorPoint) ifFalse:
		["If drag out of me, zap focus so other morphs can see drag in."
		event hand releaseMouseFocus: noteMorph]
! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/1/2007 23:58'!
noteOnSelector: aSelector
	"Sets the selector that is called when a noteOn event occurs.
	Must receive one parameter (midi note number)"
	noteOnSelector := aSelector.! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 00:24'!
setKey: aMidiKey
	"Sets the selected MIDI key"
	currentKeyMorph ifNotNil: [ currentKeyMorph color: oldColor ].
	currentKeyMorph := morphDictionary at: aMidiKey.
	currentKeyMorph ifNotNil: [
		oldColor := currentKeyMorph color.
		currentKeyMorph color: playingKeyColor.
	].

	target ifNotNil: [ target perform: noteOnSelector with: aMidiKey ].! !

!TaalPianoMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/1/2007 23:57'!
target: aTarget
	target := aTarget.! !


TaalPane subclass: #TaalPitchPane
	instanceVariableNames: 'pianoMorph baseNote fineTuneSlider fineTune'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalPitchPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 20:13'!
icon
	^ TaalBoxData iconTuning.! !

!TaalPitchPane methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:54'!
initialize
	| r c b |
	super initialize.
	
	self layoutPolicy: TableLayout new.
	self listDirection: #topToBottom;
		wrapCentering: #center;
		cellPositioning: #topCenter;
		layoutInset: 3;
		cellInset: 3.
	
	self addMorphBack: (self makeTitle: 'Pitch').

	"Setup coarse grain tuning"
	
	c := TaalPane makeColumn
		cellPositioning: #topCenter;
		cellInset: 3.
	self addMorphBack: c.
	
	r := TaalPane makeRow.
	r color: Color blue.
	c addMorphBack: r.
	
	pianoMorph := TaalPianoMorph new
		target: self;
		noteOnSelector: #setBaseNote:.
		
	r addMorphBack: pianoMorph.
	
	r := TaalPane makeRow.
	c addMorphBack: r.
		
	b := TaalPane makeTextButton: '-'.
	b target: [ self setBaseNote: baseNote-1 ]; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.
	
	b := TaalPane makeTextButton: '+'.
	b target: [ self setBaseNote: baseNote+1 ]; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.
	
	"Setup fine grain tuning"

	c := TaalPane makeColumn
		cellPositioning: #topCenter;
		cellInset: 3.
	self addMorphBack: c.
	
	c addMorphBack: (StringMorph new contents: ' ').
	c addMorphBack: (StringMorph new contents: 'Fine tuning').
	
	r := TaalPane makeRow.
	c addMorphBack: r.

	fineTuneSlider := SimpleSliderMorph new
		extent: 80 @ 2;
		target: [ :v | self setFineTune: v];
		actionSelector: #value:;
		minVal: -1.0;
		maxVal: 1.0.
	
	r addMorphBack: fineTuneSlider.
	
	r := TaalPane makeRow.
	c addMorphBack: r.
	
	b := TaalPane makeTextButton: '-'.
	b target: [ self setFineTune: fineTune-0.1 ]; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.

	b := TaalPane makeTextButton: '0'.
	b target: [ self setFineTune: 0 ]; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.

	b := TaalPane makeTextButton: '+'.
	b target: [ self setFineTune: fineTune+0.1 ]; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.

! !

!TaalPitchPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:34'!
on: aTaalBox
	super on: aTaalBox.

	self setBaseNote: 60.
	self setFineTune: 0.0.
! !

!TaalPitchPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:39'!
setBaseNote: aNote
	(baseNote = aNote) ifTrue: [ ^ self ].
	
	baseNote := aNote min: 72 max: 48.
	pianoMorph setKey: baseNote.
	taalBox setBaseNote: baseNote.
! !

!TaalPitchPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:43'!
setFineTune: aFineTune
	(fineTune = aFineTune) ifTrue: [ ^ self ].
	
	fineTune := aFineTune min: 1.0 max: -1.0.
	fineTuneSlider adjustToValue: fineTune.
	taalBox setFineTune: fineTune.
! !


TaalPane subclass: #TaalPulsePane
	instanceVariableNames: 'pulseText pulse dronePulse dronePulseSlider'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalPulsePane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 18:14'!
icon
	^ TaalBoxData iconPulse.! !

!TaalPulsePane methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 23:00'!
initialize
	| b l1 l2 l3 c r |
	super initialize.
	
	self layoutPolicy: TableLayout new.
	self listDirection: #topToBottom;
		wrapCentering: #center;
		layoutInset: 3; 
		cellInset: 3.
		
	l1 := TaalPane makeRow.
	l2 := TaalPane makeRow.
	l3 := TaalPane makeRow.
	
	l1 addMorphBack: (self makePulseButton: [ :x | (x-5)] text: '-5').
	l1 addMorphBack: (StringMorph new contents: '   ').
	l1 addMorphBack: (self makePulseButton: [ :x | (x-1)] text: '-1').
	l1 addMorphBack: (self makePulseButton: [ :x | (x+1)] text: '+1').
	l1 addMorphBack: (StringMorph new contents: '   ').
	l1 addMorphBack: (self makePulseButton: [ :x | (x+5)] text: '+5').
			
	l2 addMorphBack: (self makePulseButton: [ :x | (x-40)] text: '-40').
	l2 addMorphBack: (StringMorph new contents: '   ').
	l2 addMorphBack: (self makePulseButton: [ :x | (x-10)] text: '-10').
	l2 addMorphBack: (self makePulseButton: [ :x | (x+10)] text: '+10').
	l2 addMorphBack: (StringMorph new contents: '   ').
	l2 addMorphBack: (self makePulseButton: [ :x | (x+40)] text: '+40').
		
	l3 addMorphBack: (self makePulseButton: [ :x | (x//3)] text: '/3').
	l3 addMorphBack: (StringMorph new contents: '   ').
	l3 addMorphBack: (self makePulseButton: [ :x | (x//2)] text: '/2').
	l3 addMorphBack: (self makePulseButton: [ :x | (x*2)] text: 'x2').
	l3 addMorphBack: (StringMorph new contents: '   ').
	l3 addMorphBack: (self makePulseButton: [ :x | (x*3)] text: 'x3').
				
	pulseText := StringMorph new 
		font: (TextStyle default fontOfSize: 40);
		contents: '?'.
	
	self addMorphBack: (self makeTitle: 'Pulse').
	self addMorphBack: pulseText.
	self addMorphBack: l1.
	self addMorphBack: l2.
	self addMorphBack: l3.
	
	"Setup drone pulse control"
	
	self addMorphBack: (StringMorph new contents: ' ').

	c := TaalPane makeColumn
		cellPositioning: #topCenter;
		cellInset: 3.
	
	self addMorphBack: c.
	
	c addMorphBack: (StringMorph new contents: 'Drone pulse').
	
	dronePulseSlider := SimpleSliderMorph new
		extent: 120@16;
		minVal: 20;
		maxVal: 120;
		target: self;
		actionSelector: #setDronePulse:.
		
	c addMorphBack: dronePulseSlider.
	
	r := TaalPane makeRow.
	c addMorphBack: r.
	
	b := TaalPane makeTextButton: '-'.
	b target: [ self setDronePulse: (dronePulse - 5) ] ; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.

	b := TaalPane makeTextButton: '+'.
	b target: [ self setDronePulse: (dronePulse + 5) ]; actionSelector: #value.
	b actWhen: #whilePressed.
	r addMorphBack: b.
	! !

!TaalPulsePane methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 22:50'!
makePulseButton: aFunction text: aText
	| b |
	b := TaalPane makeTextButton: aText.
	b target: [ self setPulse: (aFunction value: pulse) ]; actionSelector: #value.
	b actWhen: #whilePressed.
	^ b.
! !

!TaalPulsePane methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 16:39'!
on: aTaalBox
	super on: aTaalBox.
	self setPulse: taalBox pulse.
	self setDronePulse: taalBox dronePulse.! !

!TaalPulsePane methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 16:32'!
setDronePulse: aPulse
	(dronePulse = aPulse) ifTrue: [ ^ self ].

	dronePulse := (aPulse asInteger) min: 240 max: 20.
	dronePulseSlider adjustToValue: dronePulse.
	taalBox setDronePulse: dronePulse.! !

!TaalPulsePane methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 19:50'!
setPulse: aPulse
	(pulse = aPulse) ifTrue: [ ^ self ].

	pulse := (aPulse asInteger) min: 320 max: 20.
	pulseText contents: (pulse asString).
	taalBox setPulse: pulse.! !


SampledInstrument subclass: #TaalSampledInstrument
	instanceVariableNames: 'name pitchBendStyle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalSampledInstrument commentStamp: 'gp 7/21/2007 00:28' prior: 0!
Subclassed to allow fine tuning.
pitchBendStyle: one of #none, #discreet, #continuous!


!TaalSampledInstrument methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:30'!
name
	^ name.! !

!TaalSampledInstrument methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:24'!
name: aName pitchBendStyle: aStyle
	name := aName.
	pitchBendStyle := aStyle.! !

!TaalSampledInstrument methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:30'!
pitchBendStyle
	^ pitchBendStyle.! !

!TaalSampledInstrument methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 15:45'!
soundForPitch: aPitch dur: d loudness: l

	| keymap note midiKey |
	l >= loudThreshold
		ifTrue: [
			d >= sustainedThreshold
				ifTrue: [keymap := sustainedLoud]
				ifFalse: [keymap := staccatoLoud]]
		ifFalse: [
			d >= sustainedThreshold
				ifTrue: [keymap := sustainedSoft]
				ifFalse: [keymap := staccatoSoft]].
	keymap ifNil: [keymap := sustainedLoud].
	
	midiKey := AbstractSound midiKeyForPitch: aPitch.
	note := (keymap at: midiKey) copy.
	^ note
		setPitch: aPitch
		dur: d
		loudness: (l * note gain)
! !


MIDIScore subclass: #TaalScore
	instanceVariableNames: 'durations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalScore methodsFor: 'as yet unclassified' stamp: 'gp 7/29/2007 20:05'!
eventForTrack: trackIndex after: eventIndex ticks: scoreTick
	| track evt idx loop sz t dur |
	
	track := tracks at: trackIndex.
	dur := durations at: trackIndex.

	sz := track size.
	loop := eventIndex // sz.
	idx := eventIndex \\ sz.
	(idx = 0) ifTrue: [ idx := sz. loop := loop - 1 ].
	
	evt := track at: idx.
	t := (loop * dur) + evt time.
	(t > scoreTick) ifTrue: [^ nil].
	
	(loop > 0) ifTrue: [
		evt := evt copy time: t.
	].

	^ evt
! !

!TaalScore methodsFor: 'as yet unclassified' stamp: 'gp 7/29/2007 20:54'!
tracks: aTracks
	| dur |
	super tracks: aTracks.
	durations := OrderedCollection new.
	tracks do: [ :track |
		dur := 0.
		track do: [:n | 
			(n isNoteEvent)
				ifTrue: [dur := dur max: n endTime]
				ifFalse: [dur := dur max: n time]
		].
		durations add: dur.
	].
! !


Object subclass: #TaalScoreInfo
	instanceVariableNames: 'name midiScore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalScoreInfo commentStamp: 'gp 7/10/2007 20:01' prior: 0!
This subclass of MIDIScore permits to seamlessly loop the score.!


!TaalScoreInfo methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 17:01'!
midiScore
	^ midiScore! !

!TaalScoreInfo methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 17:01'!
name
	^ name! !

!TaalScoreInfo methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 17:01'!
name: aName midiScore: aMidiScore
	name := aName.
	midiScore := aMidiScore.! !


TaalPane subclass: #TaalScorePane
	instanceVariableNames: 'taalSelectorMorph currentTaalButton taalButtonDict thekaSelectorMorph lehraSelectorMorph droneSelectorMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:21'!
createDroneSelector
	"Creates a control that permits to select an available drone"
	
	droneSelectorMorph := TaalScoreSelectorMorph new
		actionSelector: #setDrone:.
		
	droneSelectorMorph setTitle: 'Drone'.
		
	^ droneSelectorMorph.! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:21'!
createLehraSelector
	"Creates a control that permits to select an available lehra for the current taal"
	
	lehraSelectorMorph := TaalScoreSelectorMorph new
		actionSelector: #setLehra:.
		
	lehraSelectorMorph setTitle: 'Lehra'.
	
	^ lehraSelectorMorph.! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:21'!
createThekaSelector
	"Creates a control that permits to select an available theka for the current taal"
	
	thekaSelectorMorph := TaalScoreSelectorMorph new
		actionSelector: #setTheka:.
		
	thekaSelectorMorph setTitle: 'Theka'.
		
	^ thekaSelectorMorph.! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 19:42'!
icon
	^ TaalBoxData iconTaal.! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:09'!
initialize
	| c |
	super initialize.
	
	self layoutPolicy: TableLayout new.
	self listDirection: #topToBottom;
		wrapCentering: #topCenter;
		cellPositioning: #topCenter;
		layoutInset: 3;
		cellInset: 3;
		hResizing: #spaceFill.
		
	"Creates a control that permits to select the number of beats (rather than concrete taal).
	The actual morphs are set up in updateTaalSelector"

	self addMorphBack: (self makeTitle: 'Scores').

	taalSelectorMorph := TaalPane makeColumn
		hResizing: #spaceFill.
	self addMorphBack: taalSelectorMorph.
	
	self addMorphBack: (StringMorph new contents: ' ').

	c := TaalPane makeColumn
		hResizing: #spaceFill;
		wrapCentering: #topLeft.
		
	self addMorphBack: c.

	c addMorphBack: self createLehraSelector.
	c addMorphBack: self createThekaSelector.
	c addMorphBack: self createDroneSelector.

	
! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 8/3/2007 11:23'!
on: aTaalBox
	super on: aTaalBox.

	lehraSelectorMorph target: taalBox.
	thekaSelectorMorph target: taalBox.
	droneSelectorMorph target: taalBox.

	self updateTaalSelector.
	self setTaal: 16.	
! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/12/2007 22:13'!
setTaal: aTaal
	currentTaalButton ifNotNil: [ currentTaalButton color: TaalPane buttonColor ].
	currentTaalButton := taalButtonDict at: aTaal.
	currentTaalButton ifNotNil: [ currentTaalButton color: Color blue ].

	taalBox setTaal: aTaal.
	self updateScoreSelectors.! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 19:43'!
updateScoreSelectors
	lehraSelectorMorph on: (taalBox availableLehras).
	thekaSelectorMorph on: (taalBox availableThekas).
	droneSelectorMorph on: (taalBox availableDrones).! !

!TaalScorePane methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:52'!
updateTaalSelector
	"Creates a control that permits to select the number of beats (rather than concrete taal)"
	| col row n b |
	
	col := taalSelectorMorph.
	col removeAllMorphs.
	
	taalButtonDict := Dictionary new.	
	
	n := 0.
	
	1 to: 20 do: [ :i |
		(taalBox hasScores: i) ifTrue: [
		
			(n = 0) ifTrue: [
				row := TaalPane makeRow.
				col addMorphBack: row.
				n := 4.
			].

			b := TaalPane makeTextButton: (i asString).
			b target: self;	actionSelector: #setTaal:; arguments: (Array with: i).
			taalButtonDict at: i put: b.
			
			row addMorphBack: b.
			
			n := n-1.
		]
	].
	
	^ col
! !


ScorePlayer subclass: #TaalScorePlayer
	instanceVariableNames: 'baseNote fineTune durationMultiplier trueBend'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/1/2007 18:34'!
baseNote
	"Returns the base midi note (sa)"
	^ baseNote! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/19/2007 01:36'!
convertBend: aBend
	"Discretizes a MIDI pitch bend value if trueBend is false"
	| bend |
	trueBend ifTrue: [ ^ aBend ].
	bend := 8192 + (((aBend - 8192) // (8192//12)) * (8192//12)).
	^ bend.! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/19/2007 01:33'!
initialize
	super initialize.
	"self repeat: true."
	baseNote := 60.
	fineTune := 0.
	trueBend := false.
	durationMultiplier := 1.0.
	^ self! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 20:18'!
isDone
	^ false.! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 20:16'!
onScore: aMidiScore
	super onScore: aMidiScore.
	self reset.
	"self repeat: true."! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 01:28'!
play
	"Starts playing the current score"

	self isPlaying ifFalse: [ self resumePlaying ].
	^ self! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/19/2007 01:32'!
processNoteEventsAtTick: scoreTick
	"Process note events through the given score tick using internal Squeak sound synthesis."
	| j evt snd trk |

	1 to: score tracks size do: [:i |
		j := trackEventIndex at: i.
		[evt := score eventForTrack: i after: j ticks: scoreTick.
		 evt ~~ nil] whileTrue: [ 
			(evt isNoteEvent and: [(muted at: i) not]) ifTrue: [
				self startNote: evt forStartTick: scoreTick trackIndex: i.
			].
			
			evt isPitchBend ifTrue: [
				snd := nil.
				activeSounds do: [ :p |
					trk := p last.
					
					(trk == i) ifTrue: [
						snd := p first.
						"snd bend: evt bend."
					].
				].
				snd ifNotNil: [ snd bend: (self convertBend: evt bend) ].
			].
		
			j := j + 1.
			trackEventIndex at: i put: j]
		].
! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/1/2007 18:43'!
setBaseNote: aNote
	"Sets the base midi note (sa)"
	baseNote := aNote.! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/6/2007 01:20'!
setDurationMultiplier: aMultiplier
	"Sets a multiplier for note durations"
	durationMultiplier := aMultiplier.! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 01:44'!
setFineTune: aFineTune
	"Sets fine tuning of the sa: -1 is one semitone lower, 1 is one semitone higher, 0 is neutral"
	fineTune := aFineTune.! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 01:28'!
setInstrument: aTrackIndex to: aInstrument
	"Sets the instrument of the given track"

	self instrumentForTrack: aTrackIndex put: aInstrument.
! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/29/2007 19:15'!
setPulse: aPulse
	"Sets the current pulse in BPM"
	| wasPlaying |	

	tempo ifNil: [ ^ self ].
	wasPlaying := self isPlaying.
	wasPlaying ifTrue: [ self pause ].
	self rate: aPulse / 120.0.
	wasPlaying ifTrue: [ self resumePlaying ].
	! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/2/2007 01:29'!
setTrack: aTrackIndex to: aMIDITrack
	"Sets the contents of the given track"

	| tracks count wasPlaying |
	wasPlaying := self isPlaying.
	wasPlaying ifTrue: [ self pause ].
	tracks := self score tracks asOrderedCollection.
	count := tracks size.
	count to: aTrackIndex do: [ :i | tracks add: #() ].
	tracks at: aTrackIndex put: aMIDITrack.
	self score tracks: tracks.
	self reset.
	wasPlaying ifTrue: [ self resumePlaying ].
! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/15/2007 02:11'!
startNote: noteEvent forStartTick: startTick trackIndex: trackIndex
	"We override this method so as to handle fine tuning."

	| snd pitch note |
	
	
	(trackIndex = 4) ifTrue: [
		note := noteEvent midiKey.
		pitch := AbstractSound pitchForMIDIKey: note.
	]
	ifFalse: [
		note := (noteEvent midiKey) + (baseNote - 60).
		pitch := (AbstractSound pitchForMIDIKey: note) * (TaalBoxData pitchFactor: fineTune).
	].
	
	snd := (instruments at: trackIndex)
		soundForPitch: pitch
		dur: durationMultiplier * secsPerTick * (noteEvent endTime - startTick)
		loudness: noteEvent velocity asFloat / 127.0.
		
	snd := TaalBendableSound new on: snd pitch: pitch.		
		
	activeSounds add: (Array with: snd with: trackIndex).
! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:48'!
stop
	"Stops playing and rewinds"

	(self isPlaying) ifTrue: [ self pause].
	self reset.
	
	^ self! !

!TaalScorePlayer methodsFor: 'as yet unclassified' stamp: 'gp 7/19/2007 01:29'!
trueBend: aFlag
	"Sets the trueBend flag. If true, pitch bends are continuous, otherwise discrete."
	trueBend := aFlag.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TaalScorePlayer class
	instanceVariableNames: 'inst'!

!TaalScorePlayer class methodsFor: 'as yet unclassified' stamp: 'gp 7/10/2007 20:26'!
openScore: aFileName
	"Opens a MIDI file and returns a TaalScore"

	| score taalScore f |
	
	f := FileStream readOnlyFileNamed: aFileName.
	score := (MIDIFileReader new readMIDIFrom: f binary) asScore.
	f close.
	
	taalScore := TaalScore new
		tracks: (score tracks);
		trackInfo: (score trackInfo);
		tempoMap: (score tempoMap);
		ticksPerQuarterNote: (score ticksPerQuarterNote).
	
	^ taalScore.! !

!TaalScorePlayer class methodsFor: 'as yet unclassified' stamp: 'gp 6/30/2007 17:36'!
openTrack: aFileName
	"Opens a MIDI file and its first track"

	| score |

	score := self openScore: aFileName.
	^ score tracks at: 1.	
! !


AlignmentMorph subclass: #TaalSelectorMorph
	instanceVariableNames: 'index items nameMorph titleMorph target actionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!
!TaalSelectorMorph commentStamp: 'gp 7/21/2007 00:15' prior: 0!
Base class for morphs that permit to select items within a list.!


!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:14'!
actionSelector: aSelector
	actionSelector := aSelector.! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:15'!
decIndex
	(index = 1) 
		ifTrue: [ self setIndex: (items size) ]
		ifFalse: [ self setIndex: (index - 1) ].! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:15'!
incIndex
	(index = (items size)) 
		ifTrue: [ self setIndex: 1 ]
		ifFalse: [ self setIndex: (index + 1) ].! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:18'!
initialize
	| b c |
	super initialize.
	
	self listDirection: #leftToRight;
		wrapCentering: #topLeft;
		cellPositioning: #center;
		hResizing: #shrinkWrap;
		cellInset: 0.
		
	self color: Color white.
	
	b := TaalPane makeTextButton: '<'.
	b target: [ self decIndex. ]; actionSelector: #value. 
	self addMorphBack: b.

	b := TaalPane makeTextButton: '>'.
	b target: [ self incIndex. ]; actionSelector: #value. 
	self addMorphBack: b.

	self addMorphBack: (StringMorph new contents: ' ').

	c := TaalPane makeColumn.
	self addMorphBack: c.

	titleMorph := StringMorph new contents: '?'.
	c addMorphBack: titleMorph.
	
	nameMorph := (StringMorph contents: '?') emphasis: 2.		
	c addMorphBack: nameMorph.	
	
! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:19'!
nameFor: aItem
	"Returns the name for the given item"
	^ aItem asString.! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:16'!
on: aItemsCollection
	"Sets the items collection to choose from.".
	
	items := aItemsCollection.
	self setIndex: 1.
	
! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:17'!
setIndex: aIndex
	| item |
	index := aIndex.
	item := items at: index.
	nameMorph contents: (self nameFor: item).
	
	target ifNotNil: [ target perform: actionSelector with: item ].! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:17'!
setTitle: aTitle
	titleMorph contents: aTitle.! !

!TaalSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:14'!
target: aTarget
	target := aTarget.! !


TaalSelectorMorph subclass: #TaalInstrumentSelectorMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalInstrumentSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:39'!
nameFor: aItem
	"items are TaalSampledInstrument instances"
	^ aItem name.! !


TaalSelectorMorph subclass: #TaalScoreSelectorMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalScoreSelectorMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/21/2007 00:20'!
nameFor: aItem
	"items are TaalScore instances"
	| n |
	n := aItem name.
	^ (n subStrings: '.') at: 1.
! !


SimpleSliderMorph subclass: #TaalSliderMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalSliderMorph methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 23:34'!
initializeSlider
	super initializeSlider.
	self on: #mouseDown send: #mouseDown: to: self.
! !

!TaalSliderMorph methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 23:47'!
mouseDown: evt
	self scrollDirect: evt! !

!TaalSliderMorph methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 23:48'!
mouseMove: evt
	self scrollDirect: evt! !

!TaalSliderMorph methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 23:47'!
scrollDirect: event
	| r p |
	r _ self roomToMove.
	bounds isWide
		ifTrue: [r width = 0 ifTrue: [^ self]]
		ifFalse: [r height = 0 ifTrue: [^ self]].
	p _ event position adhereTo: r.
	self descending
		ifFalse:
			[self setValue: (bounds isWide 
				ifTrue: [(p x - r left) asFloat / r width]
				ifFalse: [(p y - r top) asFloat / r height])]
		ifTrue:
			[self setValue: (bounds isWide
				ifTrue: [(r right - p x) asFloat / r width]
				ifFalse:	[(r bottom - p y) asFloat / r height])]! !


TaalPane subclass: #TaalSystemPane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalSystemPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 20:14'!
icon
	^ TaalBoxData iconSystem.! !

!TaalSystemPane methodsFor: 'as yet unclassified' stamp: 'gp 8/18/2007 15:29'!
initialize
	| b |
	super initialize.
	
	self layoutPolicy: TableLayout new.
	self listDirection: #topToBottom;
		wrapCentering: #center;
		layoutInset: 3; 
		cellInset: 3.
	
	self addMorphBack: (StringMorph new contents: 'TaalPulse v1.1').
	
	b := SimpleButtonMorph new label: 'Restart'.
	b target: [ TaalBoxMorph2 restart ]; actionSelector: #value.
	b extent: 100@32.
	self addMorphBack: b.		

	b := SimpleButtonMorph new label: 'Quit'.
	b target: [ SmalltalkImage current quitPrimitive ]; actionSelector: #value.
	b extent: 100@32.
	self addMorphBack: b.		
! !

!TaalSystemPane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 20:14'!
on: aTaalBox
	super on: aTaalBox.
! !


RectangleMorph subclass: #TaalTestMorph
	instanceVariableNames: 'currentMorph p1 p2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalTestMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 17:54'!
initialize
	| b1 b2 |

	super initialize.
	self extent: 240 @ 320.
	
	p1 := RectangleMorph new
		color: Color red;
		extent: 208 @ 320.
		
	p1 addMorphBack: (StringMorph new contents: 'p1').
	p1 position: 32 @ 0.
	self addMorphBack: p1.
	p1 hide.

	p2 := RectangleMorph new
		color: Color blue;
		extent: 208 @ 320.

	p2 addMorphBack: (StringMorph new contents: 'p2').
	p2 position: 32 @ 0.
	self addMorphBack: p2.
	p2 hide.

	b1 := ThreePhaseButtonMorph new 
		target: [ self showMorph: p1 ];
		actionSelector: #value;
		onImage: (Form fromFileNamed: 'metronome.png');
		offImage: (Form fromFileNamed: 'metronome.png');
		position: 0 @ 0;
		extent: 32 @ 32.

	self addMorphBack: b1.
	
	b2 := ThreePhaseButtonMorph new 
		target: [ self showMorph: p2 ];
		actionSelector: #value;
		onImage: (Form fromFileNamed: 'taalBox.png');
		offImage: (Form fromFileNamed: 'taalBox.png');
		position: 0 @ 32;
		extent: 32 @ 32.
		

	self addMorphBack: b2.
! !

!TaalTestMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 17:53'!
showMorph: aMorph
	currentMorph ifNotNil: [ currentMorph hide ].
	currentMorph := aMorph. 
	currentMorph ifNotNil: [ currentMorph show ].
! !


AlignmentMorph subclass: #TaalVolumeControlMorph
	instanceVariableNames: 'value muted slider muteButton target actionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 21:35'!
actionSelector: aSelector
	actionSelector := aSelector.! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:34'!
decVolume
	(value > 0) ifTrue: [ self setVolume: (value - 0.1) ].
! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 22:34'!
incVolume
	(value < 1.0) ifTrue: [ self setVolume: (value + 0.1) ].
! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 8/9/2007 23:26'!
initialize
	| b |
	super initialize.
	
	self listDirection: #topToBottom;
		wrapCentering: #center;
		cellPositioning: #center;
		color: Color white;
		layoutInset: 0.
		
	slider := TaalSliderMorph new
		extent: 16 @ 80;
		minVal: 0.0;
		maxVal: 1.0;
		target: [ :v | self setVolume: (1.0-v) ];
		actionSelector: #value:.
	
	self addMorphBack: slider.

	b := self makeTextButton: '+'.
	b target: self; actionSelector: #incVolume.
	b actWhen: #whilePressed.
	self addMorphBack: b.

	b := self makeTextButton: '-'.
	b target: self;	actionSelector: #decVolume.
	b actWhen: #whilePressed.
	self addMorphBack: b.

	muteButton := ThreePhaseButtonMorph new 
		onImage: (TaalBoxData iconMute);
		offImage: (TaalBoxData iconUnmute);
		extent: 32@32;
		target: [ self setMute: muted not ];
		actionSelector: #value.
		
	self addMorphBack: muteButton.

	muted := false.
	self setVolume: 1.0.

! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:10'!
makeTextButton: aText
	^ TaalPane makeTextButton: aText.
! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 23:17'!
setMute: aFlag
	| vol |
	muted := aFlag.
	
	muted ifTrue: [ 
		vol := 0. 
		muteButton state: #on.
	] 
	ifFalse: [ 
		vol := value. 
		muteButton state: #off.
	].
	
	target ifNotNil: [ target perform: actionSelector with: vol ].! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/11/2007 17:39'!
setVolume: aValue
	| vol |
	value := aValue.
	slider adjustToValue: (1.0-value).
	
	muted ifTrue: [ vol := 0 ] ifFalse: [ vol := value ].
	target ifNotNil: [ target perform: actionSelector with: vol ].! !

!TaalVolumeControlMorph methodsFor: 'as yet unclassified' stamp: 'gp 7/3/2007 21:35'!
target: aTarget
	target := aTarget.! !


TaalPane subclass: #TaalVolumePane
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TaalBox'!

!TaalVolumePane methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:54'!
createVolumeControl: aSelector icon: aIcon
	"Creates a control that permits to set the volume of a channel.
	The given selector will be called on taalBox with the volume as an argument"
	| c |
	
	c := TaalPane makeColumn
		cellPositioning: #topCenter;
		layoutInset: 0.
	
	c addMorphBack: (ImageMorph new image: aIcon).
	
	c addMorphBack: (TaalVolumeControlMorph new
		target: [ :v | taalBox perform: aSelector with: v ];
		actionSelector: #value:).
		
	^ c.
		! !

!TaalVolumePane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 20:13'!
icon
	^ TaalBoxData iconVolume.! !

!TaalVolumePane methodsFor: 'as yet unclassified' stamp: 'gp 7/20/2007 21:52'!
initialize
	| c |
	super initialize.
	
	self layoutPolicy: TableLayout new.
	self listDirection: #topToBottom;
		wrapCentering: #center;
		cellPositioning: #topCenter;
		layoutInset: 3;
		cellInset: 3.

	c := TaalPane makeRow
		cellPositioning: #topCenter;
		cellInset: 10.

	c addMorphBack: (self createVolumeControl: #setVolume: icon: TaalBoxData iconVolGlobal).
	c addMorphBack: (StringMorph new contents: '    ').
	c addMorphBack: (self createVolumeControl: #setLehraVolume: icon: TaalBoxData iconVolLehra).
	c addMorphBack: (self createVolumeControl: #setThekaVolume: icon: TaalBoxData iconVolTheka).
	c addMorphBack: (self createVolumeControl: #setDroneVolume: icon: TaalBoxData iconVolDrone).
	
	self addMorphBack: (self makeTitle: 'Volume').
	self addMorphBack: c.
		
! !

!TaalVolumePane methodsFor: 'as yet unclassified' stamp: 'gp 7/8/2007 20:13'!
on: aTaalBox
	super on: aTaalBox.
! !
